{% extends "phone_auth/base.html" %}

{% block title_block %}
    User Verification
{% endblock title_block %}

{% block body_block %}
<div class="phoneauth-jumbotron">
    <div class="message">
        <h1>{{ title }}</h1>
    </div>
</div>
{% endblock body_block %}