{% extends "phone_auth/base.html" %}

{% block title_block %}
    Set Password
{% endblock title_block %}

{% block body_block %}

    <!--suppress HtmlUnknownAttribute -->
<div class="phoneauthform">
    <h3 class="phoneauthform-title">Set Password</h3>
    <form class="phoneauthform-content" method="POST">
        {% csrf_token %}
        {{ form.non_field_errors }}

        <div class="fieldWrapper">
            {{ form.new_password1.errors }}
            <label for="id_new_password1">
                New Password:
            </label>
            <input type="password"
                   id="id_new_password1"
                   name="{{ form.new_password1.html_name }}"
                   {% if form.new_password1.field.required %}required{% endif %}>
        </div>

        <div class="fieldWrapper">
            {{ form.new_password2.errors }}
            <label for="id_new_password2">
                Confirm Password:
            </label>
            <input type="password"
                   id="id_new_password2"
                   name="{{ form.new_password2.html_name }}"
                   {% if form.new_password2.field.required %}required{% endif %}>
        </div>

        <button type="submit">Submit</button>
    </form>
</div>
{% endblock body_block %}