{% extends "phone_auth/base.html" %}

{% block title_block %}
    Password Reset Completed
{% endblock title_block %}

{% block body_block %}
<div class="phoneauth-jumbotron">
    <div class="message">
        {% if title %}
            <h1>{{ title }}</h1>
        {% endif %}
    </div>
</div>
{% endblock body_block %}